#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include <ght_hash_table.h>

#include <stdarg.h>

typedef struct _CacheVal
{
    char
        ldap_server[128];   /* LDAP server user authenticated last */

    int
        ldap_port;          /* LDAP server port */

    char
        remote_ip[32];    /* remote IP in dotted quad */

    char
        cpw[128];          /* md5 crypted passwd */

    char
        user_dn[2046];

    /* authorization */
    char
        filter[512];

    int
        filter_auth_passed;

    char
        group_dn[1024];

    int
        group_auth_passed;

    char
        attr[32];

    int
        attr_auth_passed;
    /* authorization */

    time_t
        t;                 /* last time the user was authenticated */

    int
        pid;               /* the apache process PID */
}CacheVal;

static void print_cache(CacheVal *c)
{
    (void) fprintf(stderr,"ldap_server: \"%s\"\n",c->ldap_server);
    (void) fprintf(stderr,"ldap_port: %d\n",c->ldap_port);
    (void) fprintf(stderr,"remote_ip: \"%s\"\n",c->remote_ip);
    (void) fprintf(stderr,"cpw: \"%s\"\n",c->cpw);
    (void) fprintf(stderr,"user_dn: \"%s\"\n",c->user_dn);
    (void) fprintf(stderr,"filter: \"%s\"\n",c->filter);
    (void) fprintf(stderr,"filter_auth_passed: %d\n",c->filter_auth_passed);
    (void) fprintf(stderr,"group_dn: \"%s\"\n",c->group_dn);
    (void) fprintf(stderr,"group_auth_passed: %d\n",c->group_auth_passed);
    (void) fprintf(stderr,"attr: \"%s\"\n",c->attr);
    (void) fprintf(stderr,"attr_auth_passed: %d\n",c->attr_auth_passed);
    (void) fprintf(stderr,"t: %lu\n",c->t);
    (void) fprintf(stderr,"pid: %d\n",c->pid);

}


int main(int argc,char *argv[])
{
    CacheVal
        *c;

    ght_hash_table_t
        *p_table=NULL;

    int
        rc;

    char
        *user;

    c=(CacheVal *) malloc(sizeof(CacheVal));
    memset(c,0,sizeof(CacheVal));

    p_table=ght_create(1000);
    if (p_table == NULL)
    {
        (void) fprintf(stderr,"Could not create hash table\n");
        return(1);
    }

    (void) strcpy(c->ldap_server,"192.168.0.1");
    c->ldap_port=389;

    user=strdup("muquit");
    /* insert */
    rc=ght_insert(p_table,c,strlen(user),user);
    if (rc != 0)
    {
        (void) fprintf(stderr,"Could not insert %s\n",user);
        return(1);
    }

    /* retrieve */
    (void) free((char *) user);
    user=strdup("muquit");
    c=(CacheVal *) ght_get(p_table,strlen(user),user);
    if (c)
    {
        (void) fprintf(stderr,"c=%x\n",c);
        (void) fprintf(stderr,"\nRetrieved data for: %s\n\n",user);
        print_cache(c);

        (void) fprintf(stderr,"\nReplacing remote_ip,attr_auth_passed\n");
        /*
        c=(CacheVal *) ght_remove(p_table,strlen(user),user);
        */
        (void) fprintf(stderr,"Removed: %x\n",c);

        (void) strcpy(c->remote_ip,"192.168.0.207");
        c->attr_auth_passed=1;

        (void) strcpy(c->cpw,"fkdfjkdjfkdjkfjdk");
        (void) strcpy(c->group_dn,"dfdfdfdf");

        /* insert the data again */
        /*
        rc=ght_insert(p_table,c,strlen(user),user);
        if (rc != 0)
        {
            (void) fprintf(stderr,"Could not insert %s\n",user);
            return(1);
        }
        */

        /* get it again and dump */
        (void) fprintf(stderr,"Getting again\n");
        c=(CacheVal *) ght_get(p_table,strlen(user),user);
        if (c)
        {
            print_cache(c);
        }

    }

    if (p_table)
    {
        ght_finalize(p_table);
    }

    if (c)
    {
        (void) free((char *) c);
    }

    if (user)
        (void) free((char *) user);



ExitProcessing:
    return(0);
}
